@@ -1,11 +1,12 @@
 __all__ = ['User']
 
-from dataclasses import dataclass
+from dataclasses import dataclass, field
 
 from multibot import constants
 from multibot.models.database import db
 from multibot.models.enums import Platform
 from multibot.models.event_component import EventComponent
+from multibot.models.role import Role
 
 
 @dataclass(eq=False)
@@ -18,7 +19,5 @@ class User(EventComponent):
     name: str = None
     is_admin: bool = None
     is_bot: bool = None
+    roles: list[Role] = field(default_factory=list)
     original_object: constants.ORIGINAL_USER = None
-
-    def __getstate__(self):
-        return self._mongo_repr()
